"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cli_ux_1 = require("cli-ux");
const processFile_1 = require("./utils/processFile");
const processData_1 = require("./utils/processData");
const dataStream_1 = require("./utils/dataStream");
const distance_1 = require("./utils/distance");
const constant_1 = require("./constant");
async function calcDistanceChunks({ chunkData, distanceCalcInstance, fromAddr }) {
    try {
        const toAddrs = chunkData.map(item => item.destAddr || 'UNKNOW');
        const distanceResult = await distanceCalcInstance.simpleDistance({
            fromAddr,
            toAddrs
        });
        const destAddrs = distanceResult.destination_addresses;
        const originAddr = distanceResult.origin_addresses[0];
        const distElements = distanceResult.rows[0].elements;
        // write new file csv
        const chunkNewData = chunkData.map((item, index) => {
            let ggDistance = 'UNKNOWN', ggDuration = 'UNKNOWN';
            if (distElements[index] && distElements[index].status === 'OK') {
                ggDistance = distElements[index].distance.text;
                ggDuration = distElements[index].duration.text;
            }
            return Object.assign(Object.assign({}, item), { ggOriginAddr: originAddr, ggDestAddr: destAddrs[index], ggDistance,
                ggDuration });
        });
        return chunkNewData;
    }
    catch (err) {
        throw err;
    }
}
exports.main = async () => {
    return new Promise(async (resolve, reject) => {
        const inApiKey = await cli_ux_1.default.prompt(`Input the Google API Key`, { type: 'hide', required: true });
        const inFilePath = await cli_ux_1.default.prompt(`Input the path of the file CSV`, { required: true });
        const inCityName = await cli_ux_1.default.prompt(`Input the name of destination city`, { default: constant_1.DEFAULT_DEST_CITY, });
        const inOriginAddress = await cli_ux_1.default.prompt(`Input the origin address`, { default: constant_1.DEFAULT_ORIGIN_ADDR });
        const distanceCalc = new distance_1.DistanceCalculator({ apiKey: inApiKey });
        const processFile = new processFile_1.default({ filePath: inFilePath });
        const data = await processFile.readFilePromise();
        // map destination address
        const newData = data.map(item => {
            return Object.assign(Object.assign({}, item), { destAddr: `${item.wardName}, ${item.districtName}, ${inCityName}` });
        });
        const simpleBar = cli_ux_1.default.progress({
            format: 'API Progress | {bar} | {percentage}% | {value}/{total} Chunks',
            barCompleteChar: '\u2588',
            barIncompleteChar: '\u2591',
            hideCursor: true
        });
        // process chunk data
        const processData = new processData_1.default({ inputData: newData });
        const chunkData = processData.processToChunks({ maxChunk: constant_1.MAX_ELEMENTS });
        const virtualDataStream = new dataStream_1.VirtualDataStream({ chunkData });
        virtualDataStream.pushData();
        // init process bar
        simpleBar.start(chunkData.length, 0);
        let outData = [];
        virtualDataStream.on('data', async (chunk) => {
            try {
                const chunkDataCalculated = await calcDistanceChunks({ distanceCalcInstance: distanceCalc, chunkData: chunk, fromAddr: inOriginAddress });
                outData = outData.concat(chunkDataCalculated);
                simpleBar.update(dataStream_1.VirtualDataStream.pushedCount);
                // end push data stream
                if (dataStream_1.VirtualDataStream.isPushedDone) {
                    virtualDataStream.endPush();
                }
            }
            catch (error) {
                virtualDataStream.destroy();
                reject(error);
            }
        });
        virtualDataStream.on('end', async () => {
            simpleBar.stop();
            if (!outData.length) {
                console.log('Out data is empty');
                resolve(outData);
            }
            // handle write file
            try {
                console.log('Out data item: ', outData[0]);
                const header = Object.keys(outData[0]).map(value => {
                    return { id: value, title: value };
                });
                await processFile.writeFileAsync(header, outData);
                resolve(outData);
            }
            catch (error) {
                reject(error);
            }
        });
    });
};
